/* Broken Minimum Spanning Tree
 *
 * Sort all edges by weight, breaking ties by whether the edge exists in the
 * tree already or not. Then for each edge in turn, if it is not already in the
 * tree, run a DFS to find an edge that is in the tree but shouldn't be. Then
 * perform an edge swap.
 */
#include <algorithm>
#include <array>
#include <cassert>
#include <cstdint>
#include <iostream>
#include <vector>

struct DSU {
    int32_t n;
    // uf[x] = parent of x or negative size of component
    std::vector<int32_t> uf;
    DSU(int32_t n): n(n) {
        uf.assign(n, -1);
    }

    int32_t find(int32_t x) {
        return uf[x] < 0 ? x : (uf[x] = find(uf[x]));
    }

    int32_t merge(int32_t x, int32_t y) {
        int32_t xr = find(x);
        int32_t yr = find(y);
        if (xr == yr)
            return 0;

        if (uf[yr] < uf[xr]) {
            uf[yr] += uf[xr];
            uf[xr] = yr;
        } else {
            uf[xr] += uf[yr];
            uf[yr] = xr;
        }

        return 1;
    }
};

int main() {
    int32_t n, m;
    std::cin >> n >> m;

    // state: high bit: should be in tree. low bit: not in tree
    // w, state u, v, idx
    using Edge = std::array<int64_t, 5>;
    std::vector<Edge> edges(m);

    for (auto i = 0; i < m; ++i) {
        std::cin >> edges[i][2];
        --edges[i][2];
        std::cin >> edges[i][3];
        --edges[i][3];

        std::cin >> edges[i][0];
        edges[i][1] = (i >= n - 1) ? 0x1 : 0x0;

        edges[i][4] = i;
    }

    {
        std::vector<Edge> edgeCopy = edges;
        std::sort(edgeCopy.begin(), edgeCopy.end());
        DSU dsu(n);
        for (auto [w, state, u, v, idx] : edgeCopy) {
            if (dsu.merge(u, v)) {
                edges[idx][1] |= 0x2;
            }
        }
    }

    // edge indices
    std::vector<std::vector<int32_t>> graph(n);
    for (auto [w, state, u, v, idx] : edges) {
        graph[u].push_back(idx);
        graph[v].push_back(idx);
    }

    auto getV = [&edges](int64_t u, int64_t edgeIdx) -> int64_t {
        const Edge& e = edges[edgeIdx];
        return e[2] ^ e[3] ^ u;
    };

    auto dfs = [&](auto self, int64_t u, int64_t goal, int64_t par) -> std::pair<bool, int32_t> {
        if (u == goal) {
            return {true, -1};
        }

        for (auto edgeIdx : graph[u]) {
            auto v = getV(u, edgeIdx);
            if (v == par || (edges[edgeIdx][1] & 0x1)) continue;

            auto [found, out] = self(self, v, goal, u);
            if (found) {
                if (out == -1 && edges[edgeIdx][1] == 0x0) {
                    edges[edgeIdx][1] = 0x1;
                    return {found, edgeIdx};
                } else {
                    return {found, out};
                }
            }
        }

        return {false, -1};
    };

    std::vector<std::array<int64_t, 2>> ans;
    for (auto [w, state, u, v, idx] : edges) if (state == 0x3) {
        // take the path from u to v, find an edge which is in the tree but shouldn't be, and remove it.
        auto [found, out] = dfs(dfs, u, v, -1);
        assert(found && out != -1);
        ans.push_back({out, idx});
        edges[idx][1] ^= 0x1;
    }

    std::cout << ans.size() << '\n';
    for (auto [out, in] : ans) {
        std::cout << out + 1 << ' ' << in + 1 << '\n';
    }

    return 0;
}
